# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/dialoghelper',
                'doc_host': 'https://AnswerDotAI.github.io',
                'git_url': 'https://github.com/AnswerDotAI/dialoghelper',
                'lib_path': 'dialoghelper'},
  'syms': { 'dialoghelper.capture': { 'dialoghelper.capture._capture_screen': ('capture.html#_capture_screen', 'dialoghelper/capture.py'),
                                      'dialoghelper.capture.capture_screen': ('capture.html#capture_screen', 'dialoghelper/capture.py'),
                                      'dialoghelper.capture.capture_tool': ('capture.html#capture_tool', 'dialoghelper/capture.py'),
                                      'dialoghelper.capture.setup_share': ('capture.html#setup_share', 'dialoghelper/capture.py'),
                                      'dialoghelper.capture.start_share': ('capture.html#start_share', 'dialoghelper/capture.py')},
            'dialoghelper.core': { 'dialoghelper.core._add_msg_unsafe': ('core.html#_add_msg_unsafe', 'dialoghelper/core.py'),
                                   'dialoghelper.core._diff_dialog': ('core.html#_diff_dialog', 'dialoghelper/core.py'),
                                   'dialoghelper.core._find_frame_dict': ('core.html#_find_frame_dict', 'dialoghelper/core.py'),
                                   'dialoghelper.core._norm_lines': ('core.html#_norm_lines', 'dialoghelper/core.py'),
                                   'dialoghelper.core._umsg': ('core.html#_umsg', 'dialoghelper/core.py'),
                                   'dialoghelper.core.add_html': ('core.html#add_html', 'dialoghelper/core.py'),
                                   'dialoghelper.core.add_msg': ('core.html#add_msg', 'dialoghelper/core.py'),
                                   'dialoghelper.core.add_scr': ('core.html#add_scr', 'dialoghelper/core.py'),
                                   'dialoghelper.core.add_styles': ('core.html#add_styles', 'dialoghelper/core.py'),
                                   'dialoghelper.core.ast_grep': ('core.html#ast_grep', 'dialoghelper/core.py'),
                                   'dialoghelper.core.ast_py': ('core.html#ast_py', 'dialoghelper/core.py'),
                                   'dialoghelper.core.call_endp': ('core.html#call_endp', 'dialoghelper/core.py'),
                                   'dialoghelper.core.copy_msg': ('core.html#copy_msg', 'dialoghelper/core.py'),
                                   'dialoghelper.core.ctx_folder': ('core.html#ctx_folder', 'dialoghelper/core.py'),
                                   'dialoghelper.core.ctx_repo': ('core.html#ctx_repo', 'dialoghelper/core.py'),
                                   'dialoghelper.core.ctx_symfile': ('core.html#ctx_symfile', 'dialoghelper/core.py'),
                                   'dialoghelper.core.ctx_symfolder': ('core.html#ctx_symfolder', 'dialoghelper/core.py'),
                                   'dialoghelper.core.ctx_sympkg': ('core.html#ctx_sympkg', 'dialoghelper/core.py'),
                                   'dialoghelper.core.curr_dialog': ('core.html#curr_dialog', 'dialoghelper/core.py'),
                                   'dialoghelper.core.del_msg': ('core.html#del_msg', 'dialoghelper/core.py'),
                                   'dialoghelper.core.event_get': ('core.html#event_get', 'dialoghelper/core.py'),
                                   'dialoghelper.core.fc_tool_info': ('core.html#fc_tool_info', 'dialoghelper/core.py'),
                                   'dialoghelper.core.find_dname': ('core.html#find_dname', 'dialoghelper/core.py'),
                                   'dialoghelper.core.find_msg_id': ('core.html#find_msg_id', 'dialoghelper/core.py'),
                                   'dialoghelper.core.find_msgs': ('core.html#find_msgs', 'dialoghelper/core.py'),
                                   'dialoghelper.core.find_var': ('core.html#find_var', 'dialoghelper/core.py'),
                                   'dialoghelper.core.fire_event': ('core.html#fire_event', 'dialoghelper/core.py'),
                                   'dialoghelper.core.gist_file': ('core.html#gist_file', 'dialoghelper/core.py'),
                                   'dialoghelper.core.iife': ('core.html#iife', 'dialoghelper/core.py'),
                                   'dialoghelper.core.import_gist': ('core.html#import_gist', 'dialoghelper/core.py'),
                                   'dialoghelper.core.import_string': ('core.html#import_string', 'dialoghelper/core.py'),
                                   'dialoghelper.core.is_tool': ('core.html#is_tool', 'dialoghelper/core.py'),
                                   'dialoghelper.core.is_usable_tool': ('core.html#is_usable_tool', 'dialoghelper/core.py'),
                                   'dialoghelper.core.load_gist': ('core.html#load_gist', 'dialoghelper/core.py'),
                                   'dialoghelper.core.mk_toollist': ('core.html#mk_toollist', 'dialoghelper/core.py'),
                                   'dialoghelper.core.msg_del_lines': ('core.html#msg_del_lines', 'dialoghelper/core.py'),
                                   'dialoghelper.core.msg_idx': ('core.html#msg_idx', 'dialoghelper/core.py'),
                                   'dialoghelper.core.msg_insert_line': ('core.html#msg_insert_line', 'dialoghelper/core.py'),
                                   'dialoghelper.core.msg_replace_lines': ('core.html#msg_replace_lines', 'dialoghelper/core.py'),
                                   'dialoghelper.core.msg_str_replace': ('core.html#msg_str_replace', 'dialoghelper/core.py'),
                                   'dialoghelper.core.msg_strs_replace': ('core.html#msg_strs_replace', 'dialoghelper/core.py'),
                                   'dialoghelper.core.paste_msg': ('core.html#paste_msg', 'dialoghelper/core.py'),
                                   'dialoghelper.core.pop_data': ('core.html#pop_data', 'dialoghelper/core.py'),
                                   'dialoghelper.core.read_msg': ('core.html#read_msg', 'dialoghelper/core.py'),
                                   'dialoghelper.core.read_msgid': ('core.html#read_msgid', 'dialoghelper/core.py'),
                                   'dialoghelper.core.run_msg': ('core.html#run_msg', 'dialoghelper/core.py'),
                                   'dialoghelper.core.set_var': ('core.html#set_var', 'dialoghelper/core.py'),
                                   'dialoghelper.core.toggle_header': ('core.html#toggle_header', 'dialoghelper/core.py'),
                                   'dialoghelper.core.tool_info': ('core.html#tool_info', 'dialoghelper/core.py'),
                                   'dialoghelper.core.update_msg': ('core.html#update_msg', 'dialoghelper/core.py'),
                                   'dialoghelper.core.url2note': ('core.html#url2note', 'dialoghelper/core.py')},
            'dialoghelper.db_dc': {},
            'dialoghelper.inspecttools': { 'dialoghelper.inspecttools._find_frame_dict': ( 'inspecttools.html#_find_frame_dict',
                                                                                           'dialoghelper/inspecttools.py'),
                                           'dialoghelper.inspecttools.doimport': ( 'inspecttools.html#doimport',
                                                                                   'dialoghelper/inspecttools.py'),
                                           'dialoghelper.inspecttools.getdir': ('inspecttools.html#getdir', 'dialoghelper/inspecttools.py'),
                                           'dialoghelper.inspecttools.getnth': ('inspecttools.html#getnth', 'dialoghelper/inspecttools.py'),
                                           'dialoghelper.inspecttools.gettype': ( 'inspecttools.html#gettype',
                                                                                  'dialoghelper/inspecttools.py'),
                                           'dialoghelper.inspecttools.getval': ('inspecttools.html#getval', 'dialoghelper/inspecttools.py'),
                                           'dialoghelper.inspecttools.inspect_tool_info': ( 'inspecttools.html#inspect_tool_info',
                                                                                            'dialoghelper/inspecttools.py'),
                                           'dialoghelper.inspecttools.resolve': ( 'inspecttools.html#resolve',
                                                                                  'dialoghelper/inspecttools.py'),
                                           'dialoghelper.inspecttools.run_code_interactive': ( 'inspecttools.html#run_code_interactive',
                                                                                               'dialoghelper/inspecttools.py'),
                                           'dialoghelper.inspecttools.showsrc': ( 'inspecttools.html#showsrc',
                                                                                  'dialoghelper/inspecttools.py'),
                                           'dialoghelper.inspecttools.symsrc': ( 'inspecttools.html#symsrc',
                                                                                 'dialoghelper/inspecttools.py')},
            'dialoghelper.stdtools': {},
            'dialoghelper.tmux': { 'dialoghelper.tmux._pane_data': ('tmux.html#_pane_data', 'dialoghelper/tmux.py'),
                                   'dialoghelper.tmux._session_data': ('tmux.html#_session_data', 'dialoghelper/tmux.py'),
                                   'dialoghelper.tmux._ssh': ('tmux.html#_ssh', 'dialoghelper/tmux.py'),
                                   'dialoghelper.tmux._window_data': ('tmux.html#_window_data', 'dialoghelper/tmux.py'),
                                   'dialoghelper.tmux.flatten_dict': ('tmux.html#flatten_dict', 'dialoghelper/tmux.py'),
                                   'dialoghelper.tmux.list_panes': ('tmux.html#list_panes', 'dialoghelper/tmux.py'),
                                   'dialoghelper.tmux.list_sessions': ('tmux.html#list_sessions', 'dialoghelper/tmux.py'),
                                   'dialoghelper.tmux.list_windows': ('tmux.html#list_windows', 'dialoghelper/tmux.py'),
                                   'dialoghelper.tmux.pane': ('tmux.html#pane', 'dialoghelper/tmux.py'),
                                   'dialoghelper.tmux.panes': ('tmux.html#panes', 'dialoghelper/tmux.py'),
                                   'dialoghelper.tmux.sessions': ('tmux.html#sessions', 'dialoghelper/tmux.py'),
                                   'dialoghelper.tmux.set_default_history': ('tmux.html#set_default_history', 'dialoghelper/tmux.py'),
                                   'dialoghelper.tmux.shell_ret': ('tmux.html#shell_ret', 'dialoghelper/tmux.py'),
                                   'dialoghelper.tmux.tmux_tool_info': ('tmux.html#tmux_tool_info', 'dialoghelper/tmux.py'),
                                   'dialoghelper.tmux.windows': ('tmux.html#windows', 'dialoghelper/tmux.py')}}}
